/******************************************************************************
*
* Freescale Semiconductor Inc.
* (c) Copyright 2011-2012 Freescale Semiconductor, Inc.
* ALL RIGHTS RESERVED.
*
***************************************************************************//*!
*
* @file tsi.h
*
* @author a13984
*
* @version 0.0.1
*
* @date Aug 23, 2011
*
* @brief application entry point which performs application specific tasks. 
*
*******************************************************************************
*
* provide a demo for how to initialize the PT60, output messages via SCI, 
* flash operations, etc.
* NOTE:
*	printf call may occupy a lot of memory (around 1924 bytes), so please
*	consider your code size before using printf.
******************************************************************************/



#ifndef TSI_H_
#define TSI_H_
/******************************************************************************
* Includes
******************************************************************************/

/******************************************************************************
* Constants
******************************************************************************/

/******************************************************************************
* Macros
******************************************************************************/

#define EVB_BOARD			0x00
#define TOWER_BOARD			0x01

#if defined(TWR_PT60)
	#define BOARD_TYPE			TOWER_BOARD
#else
	#define BOARD_TYPE			EVB_BOARD
#endif

#if( BOARD_TYPE == TOWER_BOARD )

#define TSI_PEN0_VALUE		0x03
#define TSI_PEN1_VALUE		0xc0


#elif( BOARD_TYPE == EVB_BOARD )
#define TSI_PEN0_VALUE		0x0f
#define TSI_PEN1_VALUE		0x00
#endif

/* define prescaler of the output electrode oscillator */
#define TSI_CS1_PS_1		(0)
#define TSI_CS1_PS_2		(1)
#define TSI_CS1_PS_4        (2)
#define TSI_CS1_PS_8       	(3)
#define TSI_CS1_PS_16       (4)
#define TSI_CS1_PS_32       (5)
#define TSI_CS1_PS_64      	(6)
#define TSI_CS1_PS_128     	(7)

/* define the scan number for each electrode */
#define TSI_CS1_NSCN_1		0
#define TSI_CS1_NSCN_2		1
#define TSI_CS1_NSCN_3		2
#define TSI_CS1_NSCN_4		3
#define TSI_CS1_NSCN_5		4
#define TSI_CS1_NSCN_6		5
#define TSI_CS1_NSCN_7		6
#define TSI_CS1_NSCN_8		7
#define TSI_CS1_NSCN_9		8
#define TSI_CS1_NSCN_10		9
#define TSI_CS1_NSCN_11		10
#define TSI_CS1_NSCN_12		11
#define TSI_CS1_NSCN_13		12
#define TSI_CS1_NSCN_14		13
#define TSI_CS1_NSCN_15		14
#define TSI_CS1_NSCN_16		15
#define TSI_CS1_NSCN_17		16
#define TSI_CS1_NSCN_18		17
#define TSI_CS1_NSCN_19		18
#define TSI_CS1_NSCN_20		19
#define TSI_CS1_NSCN_21		20
#define TSI_CS1_NSCN_22		21
#define TSI_CS1_NSCN_23		22
#define TSI_CS1_NSCN_24		23
#define TSI_CS1_NSCN_25		24
#define TSI_CS1_NSCN_26		25
#define TSI_CS1_NSCN_27		26
#define TSI_CS1_NSCN_28		27
#define TSI_CS1_NSCN_29		28
#define TSI_CS1_NSCN_30		29
#define TSI_CS1_NSCN_31		30
#define TSI_CS1_NSCN_32		31

/* define the reference oscillator charge current */
#define TSI_CS2_REFCHRG_500nA		(0)<<5
#define TSI_CS2_REFCHRG_1uA			(1)<<5
#define TSI_CS2_REFCHRG_2uA			(2)<<5
#define TSI_CS2_REFCHRG_4uA			(3)<<5 
#define TSI_CS2_REFCHRG_8uA			(4)<<5 
#define TSI_CS2_REFCHRG_16uA		(5)<<5
#define TSI_CS2_REFCHRG_32uA		(6)<<5
#define TSI_CS2_REFCHRG_64uA		(7)<<5

/* define the oscillator's voltage rails */
#define TSI_CS2_DVOLT_00			(0)<<3
#define TSI_CS2_DVOLT_01			(1)<<3
#define TSI_CS2_DVOLT_10			(2)<<3
#define TSI_CS2_DVOLT_11			(3)<<3

/* define the extern oscillator charge current */
#define TSI_CS2_EXTCHRG_500nA		(0)
#define TSI_CS2_EXTCHRG_1uA			(1)
#define TSI_CS2_EXTCHRG_2uA			(2)
#define TSI_CS2_EXTCHRG_4uA			(3) 
#define TSI_CS2_EXTCHRG_8uA			(4) 
#define TSI_CS2_EXTCHRG_16uA		(5)
#define TSI_CS2_EXTCHRG_32uA		(6)
#define TSI_CS2_EXTCHRG_64uA		(7)



typedef struct
{
	unsigned short MinValue;
	unsigned short MaxValue;
	unsigned short Average;
	unsigned short Delta;
}ELECTRODE_CHECK_INFO;

#define ELECTRODE_NUM				4
#define KEY_OVERTIME				3
#define KEY_ON_SENSITIVITY			2000
#define KEY_OFF_SENSITIVITY			100
#define KEY_RELEASED				0x00
#define KEY_TOUCHED					0x01

/******************************************************************************
* Types
******************************************************************************/

typedef struct
{
	unsigned short BaseLine;
	short Delta;
	unsigned char  Channel;
	unsigned char  Count;
	unsigned char  State;
}ELECTRODE_WORK_INFO;

/******************************************************************************
* Global variables
******************************************************************************/

/******************************************************************************
* Global functions
******************************************************************************/

extern void TSI_Init( void );
extern void TSI_Processing( void );
extern void KEY1_Processing( void );
extern void KEY2_Processing( void );
extern void KEY3_Processing( void );
extern void KEY4_Processing( void );

#endif /* TSI_H_ */
